@@ -6,7 +6,12 @@ from typing import Callable, Coroutine, Dict, List, Optional, Tuple
 
 from .aioprotocol import AIOLEDENETProtocol
 from .aioscanner import AIOBulbScanner
-from .base_device import PROTOCOL_PROBES, DeviceType, LEDENETDevice
+from .base_device import (
+    ALL_ADDRESSABLE_PROTOCOLS,
+    ALL_IC_PROTOCOLS,
+    DeviceType,
+    LEDENETDevice,
+)
 from .const import (
     COLOR_MODE_CCT,
     COLOR_MODE_DIM,
@@ -31,6 +36,7 @@ from .protocol import (
     ProtocolLEDENETAddressableChristmas,
     ProtocolLEDENETOriginal,
 )
+from .scanner import FluxLEDDiscovery
 from .utils import color_temp_to_white_levels, rgbw_brightness, rgbww_brightness
 
 _LOGGER = logging.getLogger(__name__)
@@ -38,6 +44,9 @@ _LOGGER = logging.getLogger(__name__)
 
 COMMAND_SPACING_DELAY = 1
 MAX_UPDATES_WITHOUT_RESPONSE = 4
+DEVICE_CONFIG_WAIT_SECONDS = (
+    3.5  # time it takes for the device to respond after a config change
+)
 POWER_STATE_TIMEOUT = 1.2  # number of seconds before declaring on/off failed
 
 #
@@ -57,20 +66,26 @@ NEVER_TIME = -PUSH_UPDATE_INTERVAL
 class AIOWifiLedBulb(LEDENETDevice):
     """A LEDENET Wifi bulb device."""
 
-    def __init__(self, ipaddr: str, port: int = 5577, timeout: int = 5) -> None:
+    def __init__(
+        self,
+        ipaddr: str,
+        port: int = 5577,
+        timeout: float = 5,
+        discovery: Optional[FluxLEDDiscovery] = None,
+    ) -> None:
         """Init and setup the bulb."""
-        super().__init__(ipaddr, port, timeout)
+        super().__init__(ipaddr, port, timeout, discovery)
         self._lock = asyncio.Lock()
         self._aio_protocol: Optional[AIOLEDENETProtocol] = None
         self._power_restore_future: "asyncio.Future[bool]" = asyncio.Future()
-        self._ic_future: "asyncio.Future[bool]" = asyncio.Future()
+        self._device_config_lock: asyncio.Lock = asyncio.Lock()
+        self._device_config_future: asyncio.Future[bool] = asyncio.Future()
+        self._device_config_setup = False
         self._on_futures: List["asyncio.Future[bool]"] = []
         self._off_futures: List["asyncio.Future[bool]"] = []
         self._determine_protocol_future: Optional["asyncio.Future[bool]"] = None
         self._updated_callback: Optional[Callable[[], None]] = None
         self._updates_without_response = 0
-        self._pixels_per_segment: Optional[int] = None
-        self._segments: Optional[int] = None
         self._last_update_time: float = NEVER_TIME
         self._power_restore_state: Optional[PowerRestoreStates] = None
         self._buffer = b""
@@ -86,11 +101,18 @@ class AIOWifiLedBulb(LEDENETDevice):
         self._updated_callback = updated_callback
         await self._async_determine_protocol()
         assert self._protocol is not None
-        if self._protocol.zones:
-            await self._async_addressable_setup()
+        if isinstance(self._protocol, ALL_IC_PROTOCOLS):
+            await self._async_device_config_setup()
             return
         if self.device_type == DeviceType.Switch:
             await self._async_switch_setup()
+            return
+        _LOGGER.debug(
+            "%s: device_config: wiring=%s operating_mode=%s",
+            self.ipaddr,
+            self.wiring,
+            self.operating_mode,
+        )
 
     async def _async_switch_setup(self) -> None:
         """Setup a switch."""
@@ -102,17 +124,21 @@ class AIOWifiLedBulb(LEDENETDevice):
             self.set_unavailable()
             raise RuntimeError("Could not determine power restore state")
 
-    async def _async_addressable_setup(self) -> None:
+    async def _async_device_config_setup(self) -> None:
         """Setup an addressable light."""
         assert self._protocol is not None
         if isinstance(self._protocol, ProtocolLEDENETAddressableChristmas):
-            self._pixels_per_segment = 6  # currently hard coded
+            self._device_config = self._protocol.parse_strip_setting(b"")
             return
 
-        assert isinstance(self._protocol, ProtocolLEDENETAddressableA3)
+        if self._device_config_setup:
+            self._device_config_future = asyncio.Future()
+        self._device_config_setup = True
+
+        assert isinstance(self._protocol, ALL_ADDRESSABLE_PROTOCOLS)
         await self._async_send_msg(self._protocol.construct_request_strip_setting())
         try:
-            await asyncio.wait_for(self._ic_future, timeout=self.timeout)
+            await asyncio.wait_for(self._device_config_future, timeout=self.timeout)
         except asyncio.TimeoutError:
             self.set_unavailable()
             raise RuntimeError("Could not determine number pixels")
@@ -329,14 +355,14 @@ class AIOWifiLedBulb(LEDENETDevice):
         assert self._protocol is not None
         if not self._protocol.zones:
             raise ValueError("{self.model} does not support zones")
-        assert self._pixels_per_segment is not None
+        assert self._device_config is not None
         assert isinstance(
             self._protocol,
             (ProtocolLEDENETAddressableA3, ProtocolLEDENETAddressableChristmas),
         )
         await self._async_send_msg(
             self._protocol.construct_zone_change(
-                self._pixels_per_segment, rgb_list, speed, effect
+                self._device_config.pixels_per_segment, rgb_list, speed, effect
             )
         )
 
@@ -435,6 +461,58 @@ class AIOWifiLedBulb(LEDENETDevice):
             self._protocol.construct_power_restore_state_change(new_power_restore_state)
         )
 
+    async def async_set_device_config(
+        self,
+        operating_mode: Optional[str] = None,
+        wiring: Optional[str] = None,
+        ic_type: Optional[str] = None,  # ic type
+        pixels_per_segment: Optional[int] = None,  # pixels per segment
+        segments: Optional[int] = None,  # number of segments
+        music_pixels_per_segment: Optional[int] = None,  # music pixels per segment
+        music_segments: Optional[int] = None,  # number of music segments
+    ) -> None:
+        """Set device configuration."""
+        # Since Home Assistant will modify one value at a time,
+        # we need to lock, and then update so the previous value
+        # modification does not get trampled in the event they
+        # change two values before the first one has been updated
+        async with self._device_config_lock:
+            device_config = self.model_data.device_config
+            operating_mode_num = (
+                self.operating_mode_num
+                if operating_mode is None
+                else device_config.operating_mode_to_num[operating_mode]
+            )
+            wiring_num = (
+                self.wiring_num
+                if wiring is None
+                else device_config.wiring_to_num[wiring]
+            )
+            ic_type_num = (
+                self.ic_type_num
+                if ic_type is None
+                else device_config.ic_type_to_num[ic_type]
+            )
+            assert self._protocol is not None
+            assert not isinstance(self._protocol, ProtocolLEDENETOriginal)
+            await self._async_send_msg(
+                self._protocol.construct_device_config(
+                    operating_mode_num,
+                    wiring_num,
+                    ic_type_num,
+                    pixels_per_segment or self.pixels_per_segment,
+                    segments or self.segments,
+                    music_pixels_per_segment or self.music_pixels_per_segment,
+                    music_segments or self.music_segments,
+                )
+            )
+            if isinstance(self._protocol, ALL_IC_PROTOCOLS):
+                await self._async_device_config_resync()
+
+    async def _async_device_config_resync(self) -> None:
+        await asyncio.sleep(DEVICE_CONFIG_WAIT_SECONDS)
+        await self._async_device_config_setup()
+
     async def _async_connect(self) -> None:
         """Create connection."""
         _, self._aio_protocol = await asyncio.wait_for(
@@ -493,6 +571,7 @@ class AIOWifiLedBulb(LEDENETDevice):
         assert self._protocol is not None
         self.set_available()
         prev_state = self.raw_state
+        changed_state = False
         if self._protocol.is_valid_outer_message(msg):
             msg = self._protocol.extract_inner_message(msg)
 
@@ -500,13 +579,14 @@ class AIOWifiLedBulb(LEDENETDevice):
             self._async_process_state_response(msg)
         elif self._protocol.is_valid_power_state_response(msg):
             self.process_power_state_response(msg)
-        elif self._protocol.is_valid_ic_response(msg):
-            self.process_ic_response(msg)
+        elif self._protocol.is_valid_device_config_response(msg):
+            self.process_device_config_response(msg)
+            changed_state = True
         elif self._protocol.is_valid_power_restore_state_response(msg):
             self.process_power_restore_state_response(msg)
         else:
             return
-        if self.raw_state == prev_state:
+        if not changed_state and self.raw_state == prev_state:
             return
         self._process_futures_and_callbacks()
 
@@ -541,26 +621,11 @@ class AIOWifiLedBulb(LEDENETDevice):
         if not self._power_restore_future.done():
             self._power_restore_future.set_result(True)
 
-    def process_ic_response(self, msg: bytes) -> bool:
-        assert self._aio_protocol is not None
-        high_byte = msg[2]
-        low_byte = msg[3]
-        self._pixels_per_segment = (high_byte << 8) + low_byte
-        _LOGGER.debug(
-            "Pixel count (high: %s, low: %s) is: %s",
-            hex(high_byte),
-            hex(low_byte),
-            self._pixels_per_segment,
-        )
-        self._segments = msg[5]
-        _LOGGER.debug(
-            "Segment count (%s) is: %s",
-            hex(msg[5]),
-            self._segments,
-        )
-        if not self._ic_future.done():
-            self._ic_future.set_result(True)
-        return True
+    def process_device_config_response(self, msg: bytes) -> None:
+        """Process an IC (strip config) response."""
+        super().process_device_config_response(msg)
+        if not self._device_config_future.done():
+            self._device_config_future.set_result(True)
 
     async def _async_send_msg(self, msg: bytearray) -> None:
         """Write a message on the socket."""
@@ -572,7 +637,7 @@ class AIOWifiLedBulb(LEDENETDevice):
 
     async def _async_determine_protocol(self) -> None:
         # determine the type of protocol based of first 2 bytes.
-        for protocol_cls in PROTOCOL_PROBES:
+        for protocol_cls in self._protocol_probes():
             protocol = protocol_cls()
             assert isinstance(protocol, (ProtocolLEDENET8Byte, ProtocolLEDENETOriginal))
             self._protocol = protocol
